package threadedserver;

import java.net.*;
import java.io.*;
import javax.net.ssl.*;

/**
 * @author noah_singer
 *
 * To generate trust store:
 *  keytool -genkey -keystore serverexample -keyalg RSA
 *
 * To Run server as class files:
 *	1. Goto <project root>\build\classes
 *  2. generate keystore with above command
 *	3. java -Djavax.net.debug=none -Djavax.net.ssl.keyStore=serverexample -Djavax.net.ssl.keyStorePassword=student threadedserver.Main
 *
 * To Run server from jar file:
 *	1. Goto <project root>\dist
 *  2. generate keystore with above command
 *	3. java -Djavax.net.debug=none -Djavax.net.ssl.keyStore=serverexample -Djavax.net.ssl.keyStorePassword=student -jar ThreadedServer.jar
 * 
 */
public class Main {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
      System.out.println("Server ready");

			try {

				//ServerSocket ssocket = new ServerSocket( 8888 );
				SSLServerSocketFactory socketFactory = (SSLServerSocketFactory)SSLServerSocketFactory.getDefault( );
				SSLServerSocket ssocket = (SSLServerSocket)socketFactory.createServerSocket( 8888 );

				while( true ) {
					System.out.println("Listening for connection");
					SSLSocket socket = (SSLSocket)ssocket.accept( );
					new ClientHandlerThread( socket ).start( );

					System.out.println(Thread.activeCount( ) + " active threads");
				}
			} catch( IOException ioe ) {
				System.out.println(ioe);
			}
    }
}

class ClientHandlerThread extends Thread {
	private SSLSocket socket;

	public ClientHandlerThread( SSLSocket s ) {
		socket = s;
	}

	public void run( ) {
		System.out.println("New ClientHandlerThread Started");

		try {
			BufferedReader in = new BufferedReader( new InputStreamReader(socket.getInputStream( )));
			PrintWriter out = new PrintWriter( socket.getOutputStream( ) );

			while( socket.isConnected( ) ) {
				String input = in.readLine( ); //blocking
				System.out.println(input + " received from client");

				out.println( input.toUpperCase( ) );
				out.flush( );
				System.out.println("Data sent back to client");
			}
		} catch( IOException ioe ) {
			System.out.println(ioe);
		}
	}
} //end ClientHandlerThread

